/********************************** (C) COPYRIGHT *******************************
* File Name          : Main.c
* Author             : WCH
* Version            : V1.0
* Date               : 2020/12/23
* Description 		 : Main program body
* Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
* SPDX-License-Identifier: Apache-2.0
*******************************************************************************/

#include "CH56x_common.h"
#include "CH56xusb30h_LIB.h"
#include "CH56x_usb30h.h"
#include "CH56x_host_hs.h"

/* Global define */


#define BULK_IN        1
#define BULK_OUT       1
#define DUG_FUNC_EN    1

/* Global Variable */
UINT8V U30_Check_Time = 0;
volatile DevInfo g_DevInfo;
extern UINT8V g_DeviceConnectstatus;
extern UINT8V g_DeviceUsbType;

/* Function  */
void TMR0_IRQHandler( void ) __attribute__((interrupt("WCH-Interrupt-fast")));
void U30_BulkTest(void);

/*******************************************************************************
 * @fn        TMR0_IRQHandler
 *
 * @briefI    TMR0 handler.
 *
 * @return    None
 */
void TMR0_IRQHandler( void )
{
   if( R8_TMR0_INT_FLAG & RB_TMR_IF_CYC_END )
   {
      R8_TMR0_INT_FLAG = RB_TMR_IF_CYC_END;
      if( g_DeviceConnectstatus == USB_INT_CONNECT_U20 )
      {
        U30_Check_Time++;
        if( U30_Check_Time>= 10 ) //1Sĳʱʱ
        {
          g_DeviceConnectstatus = USB_INT_CONNECT;
          U30_Check_Time = 0;
          TMR0_ITCfg( DISABLE ,RB_TMR_IE_CYC_END);
          PFIC_EnableIRQ(TMR0_IRQn);
        }
      }
   }
}

/*******************************************************************************
 * @fn        DebugInit
 *
 * @briefI    nitializes the UART1 peripheral.
 *
 * @param     baudrate - UART1 communication baud rate.
 *
 * @return    None
 */
void DebugInit(UINT32 baudrate)
{
	UINT32 x;
	UINT32 t = FREQ_SYS;

	x = 10 * t * 2 / 16 / baudrate;
	x = ( x + 5 ) / 10;
	R8_UART1_DIV = 1;
	R16_UART1_DL = x;
	R8_UART1_FCR = RB_FCR_FIFO_TRIG | RB_FCR_TX_FIFO_CLR | RB_FCR_RX_FIFO_CLR | RB_FCR_FIFO_EN;
	R8_UART1_LCR = RB_LCR_WORD_SZ;
	R8_UART1_IER = RB_IER_TXD_EN;
	R32_PA_SMT |= (1<<8) |(1<<7);
	R32_PA_DIR |= (1<<8);
}
const UINT8 cmd_get_line_coding[]={0xa1,0x21,0x00,0x00,0x01,0x00,0x07,0x00};
const UINT8 cmd_set_control_line_state[]={0x21,0x22,0x00,0x00,0x01,0x00,0x00,0x00};
const UINT8 cmd_set_line_coding[]={0x21,0x20,0x00,0x00,0x01,0x00,0x07,0x00};
void cdc_class_cmd(){
    UINT32 len;
    //get line coding
    memcpy( endpTXbuff , cmd_get_line_coding , 8);
    USB30H_Send_Setup( 8 );
    len = USBSS_CtrlTransaciton(endpRXbuff);
    USB30H_Send_Status();
    if(len==7){
        printf("buad rate:%d,",*(UINT32 *)&endpRXbuff[0]);

        if(endpRXbuff[4]==0){
            printf("stop bit:1,");
        }
        else if(endpRXbuff[4]==1){
            printf("stop bit:1.5,");
        }
        else if(endpRXbuff[4]==2){
            printf("stop bit:2,");
        }

        if(endpRXbuff[5]==0){
            printf("parity type:none,");
        }
        else if(endpRXbuff[5]==1){
            printf("parity type:odd,");
        }
        else if(endpRXbuff[5]==2){
            printf("parity type:even,");
        }
        else if(endpRXbuff[5]==3){
            printf("parity type:mark,");
        }
        else if(endpRXbuff[5]==4){
            printf("parity type:space,");
        }

        printf("data bits:%d\n",endpRXbuff[6]);
    }
    else{
        printf("get line coding fail\n");
    }
//set_control_line_state
    memcpy( endpTXbuff , cmd_set_control_line_state , 8);
    USB30H_Send_Setup( 8 );
    USB30H_Send_Status();

    //set_control_line_state
        memcpy( endpTXbuff , cmd_set_line_coding , 8);
        USB30H_Send_Setup( 8 );
        *(UINT32 *)&endpTXbuff[0]=115200;
        endpTXbuff[4]=0;
        endpRXbuff[5]=0;
        endpRXbuff[6]=8;
        len = USBSS_CtrlTransaciton(endpTXbuff);
        USB30H_Send_Status();

}
/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main( )
{
    UINT8 ret;
    SystemInit(FREQ_SYS);
    Delay_Init(FREQ_SYS);
    mDelaymS(10);                        //delay
    DebugInit(921600);
    PRINT("\n\nThis is USB3.0 host program(120MHz)\n");

    PFIC_EnableIRQ(LINK_IRQn);            //enable USBSSH LINK global interrupt
    USBHS_Host_Init(ENABLE);              //USB2.0ʼ

    TMR0_TimerInit(8000000);              //3.0linkʱʱʼ
    TMR0_ITCfg( ENABLE ,RB_TMR_IE_CYC_END);
    PFIC_EnableIRQ( TMR0_IRQn );

    while(1)
    {
        if( g_DeviceConnectstatus == USB_INT_CONNECT )                 //USB豸
        {
            if( g_DeviceUsbType == USB_U30_SPEED )                     //3.0豸
            {
                USB30_Host_Enum();
                cdc_class_cmd();
                U30_BulkTest();
            }
//            else                                                       //2.0豸
//            {
//                mDelaymS(5);
//                ret = USBHS_Host_Enum( endpRXbuff );
//                if(ret != ERR_SUCCESS)
//                {
//                    printf("enumerate failed\n");
//                }
//                else {
//                }
//            }
            printf("wait_disconnect\n");
            while(g_DeviceConnectstatus == USB_INT_CONNECT)            //ȴ豸Ͽ
            {
              mDelaymS( 100 );
              if( g_DeviceUsbType == USB_U20_SPEED )
              {
                R8_USB_INT_FG = RB_USB_IF_DETECT;
                if(!(R8_USB_MIS_ST & RB_USB_ATTACH) )   break;         //ǰusb豸
              }
            }
            g_DeviceConnectstatus = 0;
            g_DeviceUsbType = 0;
            USB30H_init(DISABLE);
            USBHS_Host_Init(DISABLE);
            USBHS_Host_Init(ENABLE);                                 //豸γ 2.0ʼ
            printf("disconnect\n");
        }
        else if(R8_USB_INT_FG & RB_USB_IF_DETECT)                   //ȽUSB2.0豸ӣ֮ڽUSB3.0豸ӳʼ
        {
            R8_USB_INT_FG = RB_USB_IF_DETECT;
            if( R8_USB_MIS_ST & RB_USB_ATTACH )
            {
                printf("USB2.0 DEVICE ATTACH !\n");
                g_DeviceUsbType = USB_U20_SPEED;
                g_DeviceConnectstatus = USB_INT_CONNECT_U20;
                mDelaymS(100);
                TMR0_ITCfg( ENABLE ,RB_TMR_IE_CYC_END);
                PFIC_EnableIRQ(TMR0_IRQn);
                USB30H_init(ENABLE);                      //ʼUSB3.0Ĵ
                SetBusReset();                            //λ
            }
            else
            {
                g_DeviceConnectstatus = USB_INT_DISCONNECT;
                printf("USB2.0 DEVICE DEATTACH !\n");
            }
        }
    }
}

/*******************************************************************************
 * @fn        para_init
 *
 * @briefI    parameter initialize
 *
 * @param     pusbdev
 *
 * @return    None
 */
void para_init(void)
{
    UINT8 i;
    for(i=0;i<8;i++)
    {
        g_DevInfo.InSeqNum[i]= 0;
        g_DevInfo.OutSeqNum[i]= 0  ;          //0-7Ŷ˵0
        g_DevInfo.InMaxBurstSize[i]= 0;
        g_DevInfo.OutMaxBurstSize[i]= 0  ;    //˵ͻʼ
    }
}

/*********************************************************************
 * @fn      U30_BulkTest
 *
 * @brief   Bulk test.
 *
 * @return  none
 */
void U30_BulkTest(void)
{
    UINT8   nump,send_num,rcv_num;
    UINT16  len;
    UINT32 pack_num = 0;
    UINT32 i,t=0;;
    g_DevInfo.InSeqNum[ENDP_2]= 0;
    g_DevInfo.OutSeqNum[ENDP_2]= 0  ;          //0-7Ŷ˵0
    while(1)
    {
        t++;
//===========================OUT========================================================
       endpTXbuff[0]=t;
       endpTXbuff[1]=t;//ʣ
       USBSSH->UH_TX_DMA = (UINT32)(UINT8 *)endpTXbuff;
       send_num = 1 ;                                    //Ҫ͵İ
       len = 20;            //1024
       nump = USBSS_OUTTransaction( g_DevInfo.OutSeqNum[ENDP_2] ,send_num ,ENDP_2 ,len);
       printf("OUT nump:%d\n",nump);
       g_DevInfo.OutSeqNum[ENDP_2] ++;                                   //
       g_DevInfo.OutSeqNum[ENDP_2] &= 0x1F;                             //0~31ѭж
//===========================IN===========================================================
       USBSSH->UH_RX_DMA = (UINT32)(UINT8 *)endpRXbuff;
       rcv_num  = 1 ;                                     //Ҫȡİ
       len = USBSS_INTransaction(g_DevInfo.InSeqNum[ENDP_2], &rcv_num, ENDP_2 );         //Ӷ˵2ȡ
       printf("IN len:%d,data:",len);
       g_DevInfo.InSeqNum[ENDP_2] ++;                                   //
       g_DevInfo.InSeqNum[ENDP_2] &= 0x1F;                                               //0~31ѭж
       for(i=0;i<len;i++){
           printf("%02x ",endpRXbuff[i]);
       }
       printf("\n");
    }

}


